@@ -6,13 +6,12 @@ R0 <- function (object, ...) UseMethod("R0")
 as.epidata <- function (data, ...) UseMethod("as.epidata")
 intensityplot <- function (x, ...) UseMethod("intensityplot")
 untie <- function (x, amount, ...) UseMethod("untie")
+fixef <- function (object, ...) UseMethod("fixef")
+ranef <- function (object, ...) UseMethod("ranef")
 
 ## internal function with methods for "twinSIR" and "simEpidata"
 getModel <- function (object, ...) UseMethod("getModel")
 
-## internal function with methods for [Spatial]Polygon[s], gpc.poly, owin
-xylist <- function (object, ...) UseMethod("xylist")
-
 ## (rather internal) generic with a default and a "Spatial" method
 multiplicity <- function (x, ...) UseMethod("multiplicity")
 
@@ -22,17 +21,6 @@ multiplicity <- function (x, ...) UseMethod("multiplicity")
 if(!isGeneric("plot")) setGeneric("plot", useAsDefault=plot)
 if(!isGeneric("aggregate")) setGeneric("aggregate", useAsDefault=aggregate)
 
-## Register "owin" as class in S4 so we can define methods for it
-## Note: package "maptools" also registers "owin" as a virtual S4 class by setClass("owin")
-if (!isClass("owin")) {
-    setOldClass("owin")
-    ## on unloadNamespace("surveillance"), this leads to the Warning message:
-    ## In FUN(X[[2L]], ...) :
-    ## Created a package name, '2012-08-28 17:36:02', when none found
-    ## should the setOldClass-call better be part of .onLoad() ?
-    ## Alternative: setClass("owin") like in "maptools" would not produce this warning
-}
-
 
 
 ######################################################################
