@@ -8,8 +8,8 @@
 ### algorithm (cf. Daley & Vere-Jones, 2003, Algorithm 7.5.V.).
 ###
 ### Copyright (C) 2010-2012 Sebastian Meyer
-### $Revision: 516 $
-### $Date: 2013-02-25 22:23:29 +0100 (Mo, 25. Feb 2013) $
+### $Revision: 638 $
+### $Date: 2013-09-03 16:59:05 +0200 (Die, 03 Sep 2013) $
 ################################################################################
 
 ### CAVE:
@@ -130,31 +130,17 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     tiles <- as(tiles, "SpatialPolygons") # drop possible data attribute
                                           # (-> correct over-method)
     if (is.null(W)) {
-    	if (require("maptools") && maptools::gpclibPermitStatus()) {
-            cat("Building W as the union of 'tiles' ...\n")
-            W <- maptools::unionSpatialPolygons(tiles,
-                     IDs = rep.int(1,length(tiles@polygons)),
-                     avoidGEOS = TRUE)
-            ## ensure that W has exactly the same proj4string as tiles
-            ## since the internal CRS()-call might have modified it
-            W@proj4string <- tiles@proj4string
-        } else {
-            stop("generation of 'W' from 'tiles' requires package\n",
-                 "  \"maptools\" and \"gpclibPermit()\" therein")
-        }
+        cat("Building 'W' as the union of 'tiles' ...\n")
+    	W <- unionSpatialPolygons(tiles)
     } else {
-        stopifnot(inherits(W, "SpatialPolygons"),
-                  identical(proj4string(tiles), proj4string(W)))
+        stopifnot(inherits(W, "SpatialPolygons"), identicalCRS(tiles, W))
     }
 
-    # get (upper bound for) maximum extent of W (i.e. the diagonal of the bbox)
-    bboxW <- bbox(W)
-    maxExtentOfW <- sqrt(sum(apply(bboxW, 1, diff)^2))
-
     # Transform W into a gpc.poly
     Wgpc <- as(W, "gpc.poly")
+    maxExtentOfW <- maxExtent.gpc.poly(Wgpc)
 
-
+    
     ### Check mark-generating function
 
     # eps.t and eps.s are also unpredictable marks (generated by rmarks)
@@ -206,7 +192,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         .stillInfective <- with(events@data, time <= t0 & time + eps.t > t0)
         Nout <- sum(.stillInfective)    # = number of events in the prehistory
         if (Nout > 0L) {
-            stopifnot(identical(proj4string(events), proj4string(W)))
+            stopifnot(identicalCRS(events, W))
             events <- events[.stillInfective,]
             # check event types
             events@data$type <- factor(events@data$type, levels=typeNames)
@@ -390,8 +376,8 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         ## over the influence regions of the events
         if (!constantsiaf && !is.null(siaf$Fcircle) && !is.null(siaf$effRange))
         {
-            ## pre-compute effective range of the 'siaf' (used by .siafInt)
-            effRangeTypes <- rep(siaf$effRange(siafpars), length.out = nTypes)
+            ## pre-compute effective range of the 'siaf' (USED BY .siafInt)
+            effRangeTypes <- rep_len(siaf$effRange(siafpars), nTypes)
         }
         .siafInt <- .siafIntFUN(siaf = siaf, noCircularIR = FALSE)
                                              # not certain beforehand
@@ -463,7 +449,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     
     eTermsCalc <- function (eventData, eventCoords)
     {
-        # extract some marks from the eventData
+        # extract some marks from the eventData (USED INSIDE .siafInt() BELOW!)
         eventTypes <- as.integer(eventData$type)
         eps.s <- eventData$eps.s
         # distance to the border (required for siafInt below, and for epidataCS)
@@ -475,8 +461,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
                 data = data.frame(eps.s = eps.s, .bdist = bdist),
                 match.ID = FALSE
             ),
-            Wgpc = Wgpc,
-            npoly = nCircle2Poly
+            Wgpc = Wgpc, W = W, npoly = nCircle2Poly, maxExtent = maxExtentOfW
         ) else list()
         # epidemic terms
         if (!hase) {
@@ -492,7 +477,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
         }
         # Matrix of terms in the epidemic component
         eTerms <- cbind(
-            qSum = qSumTypes[eventData$type],
+            qSum = qSumTypes[eventTypes],
             expeta = exp(drop(mme %*% gamma)),
             siafInt = siafInts
         )
@@ -807,7 +792,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
             .sources <- infectives[eventMatrix[infectives,"type"] %in% which(qmatrix[,.eventType])]
             if (length(.sources) > 0L) {
                 .sdiffs <- .eventLocation[rep.int(1L,length(.sources)),,drop=FALSE] - eventCoords[.sources,,drop=FALSE]
-                .sources <- .sources[sqrt(rowSums(.sdiffs^2)) <= eventMatrix[.sources,"eps.s"]]
+                .sources <- .sources[sqrt(.rowSums(.sdiffs^2, length(.sources), 2L)) <= eventMatrix[.sources,"eps.s"]]
             }
 
             # calculate actual intensity at this time, location and type
@@ -937,10 +922,10 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
     events <- SpatialPointsDataFrame(
         coords = eventCoords[seqAlongEvents,,drop=FALSE], data = eventData,
         proj4string = W@proj4string, match.ID = FALSE
-        #, bbox = bboxW)   # the bbox of SpatialPoints is defined as the actual
-                           # bbox of the points and is also updated every time 
-                           # when subsetting the SpatialPoints object
-                           # -> useless to specify it as the bbox of W
+        #, bbox = bbox(W))   # the bbox of SpatialPoints is defined as the actual
+                             # bbox of the points and is also updated every time 
+                             # when subsetting the SpatialPoints object
+                             # -> useless to specify it as the bbox of W
     )
 
     if (.onlyEvents) {
@@ -957,7 +942,7 @@ simEpidataCS <- function (endemic, epidemic, siaf, tiaf, qmatrix, rmarks,
 
     cat("Done.\n")
     # append configuration of the model
-    epi$bbox <- bboxW
+    epi$bbox <- bbox(W)
     epi$timeRange <- c(t0, T)
     epi$formula <- list(
                    endemic = if (typeSpecificEndemicIntercept) {
@@ -1203,9 +1188,9 @@ plot.simEpidataCSlist <- function (x,
         if (nsim > 4) which <- sample(which, 4L)
     }
     opar <- par(mfrow = mfrow); on.exit(par(opar))
-    main <- rep(main, length.out=length(which))
-    for (i in which) {
-        do.call("plot", args=list(x=quote(x[[i]]), aggregate=aggregate,
+    main <- rep_len(main, length(which))
+    for (i in seq_along(which)) {
+        do.call("plot", args=list(x=quote(x[[which[i]]]), aggregate=aggregate,
                         subset=substitute(subset), main = main[i], ...))
     }
 }
